import { of as of$1 } from 'rxjs';
import { map as map$1, catchError, startWith } from 'rxjs/operators';
import { Component, Input, InjectionToken, ComponentFactoryResolver, Inject, ViewChild, ViewContainerRef, Pipe, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

const loadingStates = [
    'idle',
    'loading',
    'loaded',
    'error',
];

function isIdle(loadable) {
    return loadable.type === 'idle';
}
function isLoading(loadable) {
    return loadable.type === 'loading';
}
function hasErrored(loadable) {
    return loadable.type === 'error';
}
function isLoaded(loadable) {
    return loadable.type === 'loaded';
}
function isLoadable(object) {
    return (typeof object === 'object' &&
        !!object &&
        'type' in object &&
        loadingStates.includes(object.type));
}
function getLoadingState(loadable) {
    return loadable.type;
}

const idle = { type: 'idle' };
const loading = { type: 'loading' };
function loaded(value) {
    return { type: 'loaded', value };
}
function errored(error) {
    return { type: 'error', error };
}

function flatMap(f, loadable) {
    return isLoaded(loadable) ? f(loadable.value) : loadable;
}

function fold(constructors, loadable) {
    if (isLoaded(loadable)) {
        return constructors.loaded(loadable.value);
    }
    else if (hasErrored(loadable)) {
        return constructors.error(loadable.error);
    }
    else if (isIdle(loadable)) {
        return constructors.idle();
    }
    else {
        return constructors.loading();
    }
}

function fromPromise(promise) {
    return promise.then(loaded).catch(errored);
}

function of(value) {
    return loaded(value);
}

function map(f, loadable) {
    return flatMap((t) => of(f(t)), loadable);
}

function toLoadable(observable) {
    return observable.pipe(map$1(loaded), catchError((error) => of$1(errored(error))), startWith(loading));
}

class LoadableErrorComponent {
}
LoadableErrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ld-loadable-error',
                template: "<h3>Error</h3>\n<pre><code>{{ error | json }}</code></pre>\n",
                styles: [""]
            },] }
];
LoadableErrorComponent.propDecorators = {
    error: [{ type: Input }]
};

class LoadableLoadingComponent {
}
LoadableLoadingComponent.decorators = [
    { type: Component, args: [{
                selector: 'ld-loadable-loading',
                template: "<h3>Loading...</h3>\n",
                styles: [""]
            },] }
];

const defaultComponents = {
    error: LoadableErrorComponent,
    loading: LoadableLoadingComponent,
};

const DEFAULT_COMPONENTS = new InjectionToken('DEFAULT_COMPONENTS', {
    factory: () => defaultComponents,
});

function setComponentInputs(defaultComponentRef, loadable) {
    defaultComponentRef.instance.type = loadable.type;
    if (isLoaded(loadable)) {
        defaultComponentRef.instance.value = loadable.value;
    }
    else if (hasErrored(loadable)) {
        defaultComponentRef.instance.error = loadable.error;
    }
}
function loadableTemplateContext(loadable) {
    return fold({
        idle: () => ({}),
        loading: () => ({}),
        loaded: (value) => ({ value }),
        error: (error) => ({ error }),
    }, loadable);
}
class LoadableComponent {
    constructor(resolver, defaultComponents) {
        this.resolver = resolver;
        this.defaultComponents = defaultComponents;
        this.loadable = idle;
        this.templates = {};
        this.showNgContent = false;
    }
    get loadingState() {
        return getLoadingState(this.loadable);
    }
    updateContent() {
        if (this.loadingState === 'loaded') {
            return;
        }
        this.templateRef = this.templates[this.loadingState];
        const defaultComponent = this.defaultComponents[this.loadingState];
        if (!this.templateRef && defaultComponent && this.content) {
            this.content.clear();
            const factory = this.resolver.resolveComponentFactory(defaultComponent);
            this.defaultComponentRef = this.content.createComponent(factory);
            setComponentInputs(this.defaultComponentRef, this.loadable);
        }
    }
    ngOnChanges(changes) {
        var _a;
        if ('loadable' in changes) {
            this.showNgContent = isLoaded(this.loadable);
            this.templateContext = Object.assign(Object.assign({}, loadableTemplateContext(this.loadable)), { type: this.loadable.type });
        }
        if ('loadable' in changes || 'templates' in changes) {
            (_a = this.defaultComponentRef) === null || _a === void 0 ? void 0 : _a.destroy();
            this.updateContent();
        }
    }
    ngAfterViewInit() {
        setTimeout(() => this.updateContent());
    }
}
LoadableComponent.decorators = [
    { type: Component, args: [{
                selector: 'ld-loadable',
                template: "<ng-content *ngIf=\"showNgContent\"></ng-content>\n<ng-container *ngIf=\"!showNgContent\">\n  <ng-container\n    *ngTemplateOutlet=\"\n      templateRef ? templateRef : content;\n      context: templateContext\n    \"\n  ></ng-container\n></ng-container>\n<ng-template #content></ng-template>\n",
                styles: [""]
            },] }
];
LoadableComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: undefined, decorators: [{ type: Inject, args: [DEFAULT_COMPONENTS,] }] }
];
LoadableComponent.propDecorators = {
    loadable: [{ type: Input }],
    templates: [{ type: Input }],
    content: [{ type: ViewChild, args: ['content', { read: ViewContainerRef },] }]
};

class LoadableError extends Error {
    constructor(message) {
        super(message);
        this.message = message;
        this.name = 'LoadableError';
    }
}

class IsLoadingPipe {
    transform(object) {
        if (!isLoadable(object)) {
            throw new LoadableError('The passed object is not a loadable');
        }
        return isLoading(object);
    }
}
IsLoadingPipe.decorators = [
    { type: Pipe, args: [{
                name: 'isLoading',
            },] }
];

class LoadedValuePipe {
    transform(object) {
        if (!isLoadable(object)) {
            throw new LoadableError('The passed object is not a loadable');
        }
        return isLoaded(object) ? object.value : undefined;
    }
}
LoadedValuePipe.decorators = [
    { type: Pipe, args: [{
                name: 'loadedValue',
            },] }
];

class LoadingStatePipe {
    transform(object) {
        if (!isLoadable(object)) {
            throw new LoadableError('The passed object is not a loadable');
        }
        return getLoadingState(object);
    }
}
LoadingStatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'loadingState',
            },] }
];

class LoadingCustomizerModule {
    static forRoot(options = {}) {
        return {
            ngModule: LoadingCustomizerModule,
            providers: [
                {
                    provide: DEFAULT_COMPONENTS,
                    useValue: Object.assign(Object.assign({}, defaultComponents), options.defaultComponents),
                },
            ],
        };
    }
}
LoadingCustomizerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    IsLoadingPipe,
                    LoadedValuePipe,
                    LoadingStatePipe,
                    LoadableComponent,
                    LoadableErrorComponent,
                    LoadableLoadingComponent,
                ],
                imports: [CommonModule],
                exports: [
                    IsLoadingPipe,
                    LoadedValuePipe,
                    LoadingStatePipe,
                    LoadableComponent,
                    LoadableErrorComponent,
                    LoadableLoadingComponent,
                ],
                entryComponents: [LoadableLoadingComponent, LoadableErrorComponent],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { DEFAULT_COMPONENTS, IsLoadingPipe, LoadableComponent, LoadableError, LoadableErrorComponent, LoadableLoadingComponent, LoadedValuePipe, LoadingCustomizerModule, LoadingStatePipe, defaultComponents, errored, flatMap, fold, fromPromise, getLoadingState, hasErrored, idle, isIdle, isLoadable, isLoaded, isLoading, loaded, loading, loadingStates, map, of, setComponentInputs, toLoadable };
//# sourceMappingURL=ngx-loading-customizer.js.map
